package ru.bitel.bgbilling.modules.voice.dyn.mediator.m200;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Calendar;
import java.util.Date;

import ru.bitel.bgbilling.apps.voice.accounting.mediation.AbstractMediator;
import ru.bitel.bgbilling.apps.voice.accounting.mediation.VoiceRecord;
import ru.bitel.bgbilling.apps.voice.accounting.mediation.VoiceRecordProcessor;
import ru.bitel.common.TimeUtils;
import ru.bitel.common.Utils;

/**
 * Медиатор m200. Берет из папки файлы вида
сdr_log_01_06_2017.log
cdr_log_02_06_2017.log
cdr_log_03_06_2017.log
cdr_log_04_06_2017.log
cdr_log_05_06_2017.log
cdr_log_06_06_2017.log
cdr_log_07_06_2017.log

И помещает их в 0-вой час каждого дня 
 * 
 */
public class M200Mediator
    extends AbstractMediator
{
    private static final String FILE_PREFIX = "cdr_log_";

    @Override
    public void readHourDataLog( VoiceRecordProcessor processor, Date hour )
        throws Exception
    {
        InputStream is = null;
        BufferedReader reader = null;
        String path = device.getLogPath();
        Calendar calHour = TimeUtils.convertDateToCalendar( hour );
        
        //обрабаьываем запросы только на нулевой час 
        if ( calHour.get( Calendar.HOUR_OF_DAY ) != 0 )
        {
            return;
        }
        
        try
        {
            String fullPath = path + "/" + FILE_PREFIX +  TimeUtils.format( hour, "dd_MM_yyyy" ) + ".log";
            
            //TODO переделать чтение на java.nio2 
            File file = new File( fullPath );
            if ( file.exists() )
            {
               is = new FileInputStream( file );
            }
            
            if ( is == null )
            {
                logger.warn( "log for " + hour +  " is empty" );
                return;
            }
    
            reader = new BufferedReader( new InputStreamReader( is ), 128 * 1024 );

            //final Pattern pattern = Pattern.compile( "\\t" );

            String line;
            while( (line = reader.readLine()) != null )
            {
                String[] params = line.split( " " );

                if( params.length == 11 )
                {
                    /*if( logger.isDebugEnabled() )
                    {
                        logger.debug( "LINE: " + line );
                    }*/
                    processLine( processor, params );
                }
                else
                {
                    logger.warn( "Skip line: " + line );
                }
            }
        }
        finally
        {
            if ( reader != null )
            {
                reader.close();
            }
            
            if ( is != null )
            {
                is.close();
            }
        }       
    }

    protected void processLine( final VoiceRecordProcessor processor, final String[] params )
        throws InterruptedException
    {
        String e164CallingStationId = params[4];
        String e164CalledStationId = params[5];
        String ansCode =  params[10];
        
        // отбрасывать можно звонки с кодом ответа 1
        if ( ansCode.equals( "1" ) || ansCode.equals( "27" ) || ansCode.equals( "34" ) ||  ansCode.equals( "134" ))
        {
            return;
        }
            
        if ( e164CallingStationId.equals( "-") || e164CalledStationId.equals( "-" )  )
        {
            //игнорируем такие звонки. АТС их не пропустила
            return;
        }

        String dateTime = params[6] + " " + params[7];        
        
        VoiceRecord record = processor.next();
        record.sessionStart = TimeUtils.parseDate( dateTime, "dd-MM-yy HH:mm:ss" );        
        record.connectionDuration = Utils.parseInt( params[8], 0 );        
        record.duration = Utils.parseInt( params[9], 0 );
        record.callingStationId = params[1].replaceAll( "[^0-9]*", "" );
        record.e164CallingStationId = callingStationIdToE164( e164CallingStationId );
        record.calledStationId = params[2].replaceAll( "[^0-9]*", "" );        
        record.e164CalledStationId = calledStationIdToE164( e164CalledStationId );
        record.trunkIncoming = params[0];
        record.trunkOutgoing = params[3];
    }   
}
